def convert_cnum_to_alpha(source_maybe_cnum: int | str, check_banned_symbols: bool = False) -> str: ...
def convert_cnum_to_numeric(source_maybe_cnum: int | str, check_banned_symbols: bool = False) -> int: ...
